//
//  BJPUVideoCatalogueCell.swift
//  BJVideoPlayerUI
//
//  Created by 凡义 on 2023/5/19.
//  Copyright © 2023 BaijiaYun. All rights reserved.
//

import Foundation
import BJLiveBase

@objc
public class BJPUVideoCatalogueCell: UITableViewCell {
    let titleLabel = UILabel()
    let timeLabel = UILabel()
    let videoImageView = UIImageView()

    override public init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
        super.init(style: style, reuseIdentifier: reuseIdentifier)
        self.selectionStyle = .none
        makeSubViews()
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override public func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)
        if selected {
            contentView.backgroundColor = .bjl_color(hex: 0x1E5EFF)
        }
        else {
            contentView.backgroundColor = .bjl_color(hex: 0xC7D0E6, alpha: 0.1)
        }
    }
    
    @objc public func update(model: BJVVideoCatalogueItem) {
        titleLabel.text = model.title
        timeLabel.text = model.timeOffset
        videoImageView.bjl_setImage(with: URL(string: model.imageString))
    }
}

fileprivate extension BJPUVideoCatalogueCell {
    func makeSubViews() {
        contentView.layer.cornerRadius = 4
        contentView.layer.masksToBounds = true

        contentView.backgroundColor = .bjl_color(hex: 0xC7D0E6, alpha: 0.1)
        backgroundColor = .clear
        contentView.layer.cornerRadius = 4
        contentView.layer.masksToBounds = true

        titleLabel.textColor = .white
        titleLabel.font = .systemFont(ofSize: 12)
        titleLabel.textAlignment = .left
        titleLabel.numberOfLines = 2
        
        timeLabel.textAlignment = .right
        timeLabel.textColor = .white
        timeLabel.font = .systemFont(ofSize: 12)
        
        videoImageView.layer.cornerRadius = 4
        videoImageView.layer.masksToBounds = true
        
        videoImageView.layout(in: contentView) { make in
            make.left.equalToSuperview().offset(6)
            make.right.bottom.equalToSuperview().offset(-6)
            make.height.equalTo(videoImageView.snp.width).multipliedBy(9.0 / 16.0)
        }
        titleLabel.layout(in: contentView) { make in
            make.left.equalTo(videoImageView)
            make.bottom.equalTo(videoImageView.snp.top).offset(-6)
            make.top.equalToSuperview().offset(6)
        }
        timeLabel.layout(in: contentView) { make in
            make.centerY.equalTo(titleLabel)
            make.right.equalToSuperview().offset(-6)
            make.width.equalTo(60)
            make.left.equalTo(titleLabel.snp.right)
        }
    }
}
