//
//  QuizViewController.swift
//  BJLiveBase-BJLLocalize
//
//  Created by ney on 2022/10/20.
//  Copyright © 2022 BaijiaYun. All rights reserved.
//

import UIKit
import BJLiveBase
import BJSwiftBase
import SnapKit
import ExtSwift
import ExCodable
import BJVideoPlayerCore

public class QuizViewController: UIViewController {
    // MARK: - open
    @objc public required init(playerManager: BJVPlayerManager) {
        self.playerManager = playerManager
        super.init(nibName: nil, bundle: nil)
        addObserverForPlayer(self.playerManager)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    @objc public func update(vid: String, token: String, userName: String, userNumber: String) {
        self.userNameInner = userName
        self.userNumberInner = userNumber
        self.vid = vid
        self.token = token
        BJVQuizManager.getQuizList(withVideoVID: vid, userNumber: self.userNumberInner, token: token) { [weak self] task, quizList, error in
            guard let self = self, let quizList = quizList, quizList.count > 0 else { return }
            DispatchQueue.main.async {
                self.updateDataSource(quizList)
            }
        }
    }
    
    @objc public func playerShouldSeekTo(timeOffset: TimeInterval) {
        for i in self.dataSource where i.timeOffset >= timeOffset {
            i.didShown = false
        }
    }
    
    public override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    @objc
    public var userName: String {
        self.userNameInner
    }
    
    @objc
    public var userNumber: String {
        self.userNumberInner
    }
    
    @objc public var parentView: UIView?
    
    // MARK: - private
    private let playerManager: BJVPlayerManager
    private var userNameInner = ""
    private var userNumberInner = ""
    private var vid = ""
    private var token = ""
    private var dataSource = [BJVQuizItem]()
    private var currentQuizItem: BJVQuizItem?
    
    private let topContainerView = UIView { view in
        view.backgroundColor = .clear
    }
    private let quizContentContainerView = UIView { view in
        view.backgroundColor = .white
        view.layer.cornerRadius = 8
        view.layer.shadowRadius = 8
        view.layer.shadowOpacity = 1
        view.layer.shadowColor = UIColor.bjl_color(withHexString: "#0C2035")?.withAlphaComponent(0.16).cgColor
        view.layer.shadowOffset = .init(width: 0, height: 3)
    }
    private let topBar = TitleView { view in
    }
    private let quizView = QuizView { view in
    }
    private let quizAnswerResultView = QuizAnswerResultView { view in
    }
    private let bottomBar = BottomView { view in
    }
    
    var didSetupUI = false
    func setupUIIfNeeded() {
        if didSetupUI { return }
        didSetupUI = true
        
        view.backgroundColor = .clear
        
        topContainerView.layout(in: view) { make in
            make.centerX.equalToSuperview()
            make.centerY.equalTo(view.snp.bottom).multipliedBy(0.5)
            make.width.equalToSuperview().multipliedBy(0.7)
        }
        quizContentContainerView.layout(in: topContainerView) { make in
            make.edges.equalToSuperview()
        }
        topBar.layout(in: quizContentContainerView) { make in
            make.left.top.right.equalToSuperview()
            make.height.equalTo(UIDevice.isPad ? 44 : 32)
        }
        bottomBar.layout(in: quizContentContainerView) { make in
            make.left.bottom.right.equalToSuperview()
            make.height.equalTo(UIDevice.isPad ? 68 : 52)
        }
        replaceContentWithQuizView()
        setupEventHandler()
    }
    
    func replaceContentWithQuizView() {
        quizAnswerResultView.removeFromSuperview()
        quizView.layout(in: quizContentContainerView) { make in
            make.top.equalTo(topBar.snp.bottom)
            make.left.right.equalToSuperview()
            make.bottom.equalTo(bottomBar.snp.top)
        }
    }
    
    func setupEventHandler() {
        bottomBar.leftButtonEventHandler = { [weak self] _, _ in
            guard let self = self else { return }
            self.hideQuizVC()
            self.playerManager.play()
            
            self.reportAnswerResult(skip: true)
        }
        bottomBar.rightButtonEventHandler = { [weak self] _, _ in
            guard let self = self else { return }
            if self.quizAnswerResultView.didShown() {
                if !self.quizAnswerResultView.result {
                    // back
                    let backInterval = self.currentQuizItem?.backInterval ?? 0
                    if backInterval > 0 {
                        let newTimeOffset = max(0, self.playerManager.currentTime - backInterval)
                        self.playerShouldSeekTo(timeOffset: newTimeOffset)
                        self.playerManager.seek(newTimeOffset)
                    }
                }
                self.hideQuizVC()
                self.playerManager.play()
            }
            else {
                if self.quizView.quizOptionsView.hasNotSelect() { return }
                
                self.showResultView()
                self.bottomBar.updateRightButtonTitle(BottomView.finishButtonTitle)
                
                self.reportAnswerResult(answerIDs: self.quizView.quizOptionsView.answerIDs(),
                                        correct: self.quizAnswerResultView.result,
                                        skip: false)
            }
        }
    }
    
    func reportAnswerResult(answerIDs: String = "", correct: Bool = false, skip: Bool) {
        guard let quizData = self.currentQuizItem else { return }
        BJVQuizManager.reportQuizResult(withQuizID: String(quizData.quizID), answerIDs: skip ? "-1" : answerIDs, correct: correct, vID: self.vid, userName: self.userNameInner, userNumber: self.userNumberInner, token: self.token) { task, error in
        }
    }
    
    func addObserverForPlayer(_ player: BJVPlayerManager) {
        bjl_kvo(BJLMakeProperty(player, #selector(getter: player.currentTime)), observer: KVO(type: TimeInterval.self) { [weak self] _, _, _ in
            guard let self = self else { return .stop }
            
            self.showQuizVCIfNeeded()
            
            return .keep
        })
        
        bjl_kvo(BJLMakeProperty(player, #selector(getter: player.playStatus)), observer: KVO(type: BJVPlayerStatus.self) { [weak self] _, _, _ in
            guard let self = self else { return .stop }
            
            if self.playerManager.playStatus == .reachEnd {
                for i in self.dataSource {
                    i.didShown = false
                }
            }
            
            return .keep
        })
    }
}

// MARK: - QuizViewController: ui update
fileprivate extension QuizViewController {
    func updateDataSource(_ dataSource: [BJVQuizItem]) {
        self.dataSource.removeAll()
        self.dataSource.append(contentsOf: dataSource)
    }
    
    func showQuizVCIfNeeded() {
        guard UIApplication.shared.applicationState == .active,
              parentView != nil,
              dataSource.count > 0,
              let quiz = findQuizItemBy(timeOffset: self.playerManager.currentTime),
              !quiz.didShown
        else { return }
        
        self.currentQuizItem = quiz
        self.currentQuizItem?.didShown = true
        setupUIIfNeeded()
        parentView?.addSubview(view)
        quizView.updateOptionsUI(quizData: quiz)
        bottomBar.showLeftButton(quiz.canSkip)
        view.layout { make in
            make.edges.equalToSuperview()
        }
        self.playerManager.pause()
    }
    
    func hideQuizVC() {
        bjl_removeFromParentViewControllerAndSuperiew()
        replaceContentWithQuizView()
        bottomBar.updateRightButtonTitle(BottomView.commitButtonTitle)
    }
    
    func findQuizItemBy(timeOffset: TimeInterval) -> BJVQuizItem? {
        for i in self.dataSource {
            if abs(i.timeOffset - timeOffset) < 1 {
                return i
            }
        }
        return nil
    }

    func showResultView() {
        quizView.removeFromSuperview()
        if let data = self.currentQuizItem {
            quizAnswerResultView.updateResult(selectionIsRight(), data: data)
        }
        quizAnswerResultView.layout(in: quizContentContainerView) { make in
            make.top.equalTo(topBar.snp.bottom)
            make.left.right.equalToSuperview()
            make.bottom.equalTo(bottomBar.snp.top)
        }
    }
    
    func selectionIsRight() -> Bool {
        guard let quiz = self.currentQuizItem else { return false }
                
        for i in quiz.options where i.isCorrect != i.selected {
            return false
        }
        return true
    }
}
