//
//  BJPUViewController+observer.m
//  BJVideoPlayerUI
//
//  Created by HuangJie on 2018/4/8.
//

#import "BJPUViewController+observer.h"
#import "BJPUViewController+protected.h"
#import <BJPlayerUIBase/BJPUStopPlayAlertView.h>

@implementation BJPUViewController (observer)

- (void)setupObservers {
    bjl_weakify(self);
    // 播放状态
    [self bjl_kvo:BJLMakeProperty(self.playerManager, playStatus)
        filter:^BOOL(NSNumber *_Nullable now, NSNumber *_Nullable old, BJLPropertyChange *_Nullable change) {
            return (old == nil) || (now.integerValue != old.integerValue);
        } observer:^BOOL(NSNumber *_Nullable now, NSNumber *_Nullable old, BJLPropertyChange *_Nullable change) {
            bjl_strongify(self);
            BJVPlayerStatus status = self.playerManager.playStatus;
            NSLog(@"BJPUViewController - playerPlayStateChanged:%td", status);
            [self updateWithPlayState:status];

            // debug info
            switch (status) {
                case BJVPlayerStatus_unload:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Unload"));
                    break;

                case BJVPlayerStatus_loading:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Loading"));
                    break;

                case BJVPlayerStatus_stalled:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Stalled"));
                    break;

                case BJVPlayerStatus_ready:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Ready"));
                    [self makeLamp];
                    break;

                case BJVPlayerStatus_playing:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Playing"));
                    break;

                case BJVPlayerStatus_paused:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Paused"));
                    break;

                case BJVPlayerStatus_stopped:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Stopped"));
                    break;

                case BJVPlayerStatus_reachEnd:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_ReachEnd"));
                    break;

                case BJVPlayerStatus_failed:
                    NSLog(BJLLocalizedString(@"播放状态变化：BJVPlayerStatus_Failed"));
                    break;

                default:
                    break;
            }

            if (status == BJVPlayerStatus_loading
                || status == BJVPlayerStatus_stalled) {
                [BJLProgressHUD bjp_showLoading:BJLLocalizedString(@"正在加载") toView:self.view];
            }
            else {
                [BJLProgressHUD bjp_closeLoadingView:self.view];
            }

            if (status == BJVPlayerStatus_reachEnd && self.videoOptions.autoPlayNextAlbumVideo) {
                if (self.videoAlbumList.count > 0) {
                    self.currentAlbumIndex = self.currentAlbumIndex + 1;
                    if (self.currentAlbumIndex >= [self.videoAlbumList count]) {
                        self.currentAlbumIndex = 0;
                    }
                    [self playWithVideoInfo:self.videoAlbumList[self.currentAlbumIndex]];
                }
            }
            return YES;
        }];

    // 清晰度
    [self bjl_kvo:BJLMakeProperty(self.playerManager, currDefinitionInfo)
         observer:^BOOL(id _Nullable now, id _Nullable old, BJLPropertyChange *_Nullable change) {
             bjl_strongify(self);
             // 显示当前清晰度
             [self.mediaControlView updateWithDefinition:self.playerManager.currDefinitionInfo.definitionName];

             // 显示/隐藏 纯音频占位图
             self.audioOnlyImageView.hidden = !self.playerManager.currDefinitionInfo.isAudio;

             // 视频宽高比
             BJVDefinitionInfo *definitionInfo = self.playerManager.currDefinitionInfo;
             CGFloat width = definitionInfo.width;
             CGFloat height = definitionInfo.height;
             if (width > 0.0 && height > 0.0) {
                 CGFloat videoRatio = width / height;
                 // 更新播放视图宽高比
                 [self updatePlayerViewConstraintWithVideoRatio:videoRatio];
             }
             return YES;
         }];

    // 倍速
    [self bjl_kvo:BJLMakeProperty(self.playerManager, rate)
         observer:^BOOL(id _Nullable now, id _Nullable old, BJLPropertyChange *_Nullable change) {
             bjl_strongify(self);
             NSString *rateString = [NSString stringWithFormat:@"%.2f", self.playerManager.rate];
             [self.mediaControlView updateWithRate:rateString];
             return YES;
         }];

    [self bjl_kvo:BJLMakeProperty(self.playerManager, playInfo)
         observer:^BJLControlObserving(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
             bjl_strongify(self);
             if (self.playerManager.playInfo) {
                 [self.backButton bjl_setTitle:self.playerManager.playInfo.title forState:UIControlStateNormal possibleStates:UIControlStateHighlighted];
                 [self.mediaControlView updateSubtitleExist:self.playerManager.playInfo.subtitleInfo.count];
                 [self updateVSubtitleButtonConstriants:self.playerManager.playInfo.subtitleInfo.count];
                 // 如果 playInfo 里面有三分屏的信息，就需要把布局切换成三分屏布局
                 [self updateConstriantsWithTripartiteLayoutInfo:self.playerManager.playInfo.tripartiteLayoutInfo];
                 
                 if (self.playerManager.playInfo.tripartiteLayoutInfo == nil) {
                     [self.playerManager getVideoCatalogueInfoWithVideoVID:self.videoID token:self.token completion:^(NSURLSessionDataTask * _Nullable task, NSArray<BJVVideoCatalogueItem *> * _Nullable list, NSError * _Nullable error) {
                         bjl_strongify(self);
                         [self setupCatalogueView:list];
                     }];
                 }
                 
                 if (!self.playerManager.playInfo.tripartiteLayoutInfo
                     && self.playerManager.playInfo.enableVideoComment) {
                     [self updateConstriantsForCommentsVC:self.playerManager.playInfo.enableVideoComment];
                 }
                 else  {
                     // 如果 playInfo 里面有三分屏的信息，就需要把布局切换成三分屏布局 
//                     [self updateConstriantsWithTripartiteLayoutInfo:self.playerManager.playInfo.tripartiteLayoutInfo];
                 }
                 
                 [self updateConstriantsByPrefersLayoutTypeIfNeeded];
                 return YES;
             }
             return YES;
         }];

    // 字幕
    [self bjl_kvo:BJLMakeProperty(self.playerManager, currentSubtitle)
         observer:^BJLControlObserving(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
             bjl_strongify(self);
             self.subtitleLabel.text = self.playerManager.currentSubtitle.content;
             return YES;
         }];

    // 播放出错
    [self bjl_observe:BJLMakeMethod(self.playerManager, video:playFailedWithError:)
             observer:^BOOL(BJVPlayInfo *playInfo, NSError *error) {
                 bjl_strongify(self);
                 [self video:playInfo playFailedWithError:error];
                 return YES;
             }];

    // 播放时间
    [self bjl_kvoMerge:@[BJLMakeProperty(self.playerManager, duration),
        BJLMakeProperty(self.playerManager, currentTime)]
              observer:^(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
                  bjl_strongify(self);
                  self.currentMaxWatchTime = MAX(self.playerManager.currentTime, self.currentMaxWatchTime);
        
                  bjl_dispatch_async_main_queue(^{
                      // 设置最大拖拖动进度条时间，之后不允许更改倍速更改时间
                      if (self.playerManager.currentTime >= self.videoOptions.maxWatchTime && self.videoOptions.maxWatchTime > 0) {
                          self.sliderView.slideEnabled = NO;
                          [self.mediaControlView setSlideEnable:NO];
                          [self.playerManager setRate:1.0];
                      }
                      
                      //通知主线程刷新
                      [self.mediaControlView updateProgressWithCurrentTime:self.playerManager.currentTime
                                                             cacheDuration:self.playerManager.cachedDuration
                                                             totalDuration:self.playerManager.duration];
                      
                      if (self.videoOptions.auditionTime > 0 && self.playerManager.currentTime >= self.videoOptions.auditionTime) {
                          [self showAuditionTimeStopPlayView];
                      }
                  });
              }];

    [self.screenRecordingNotifier startObserve];
    
    [self bjl_kvo:BJLMakeProperty(self.mediaControlView, controlsHidden)
           filter:^BOOL(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
        return [value integerValue] != [oldValue integerValue];
    }
         observer:^BJLControlObserving(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
        bjl_strongify(self);
        self.topBarView.hidden = self.mediaControlView.controlsHidden;
        return YES;
    }];
    
    [self bjl_kvo:BJLMakeProperty(self.videoContentContainer, bounds)
           filter:^BOOL(NSValue  * _Nullable value, NSValue  * _Nullable oldValue, BJLPropertyChange *_Nullable change) {
        return !CGSizeEqualToSize([value CGRectValue].size, [oldValue CGRectValue].size);
    }
         observer:^BJLControlObserving(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
        bjl_strongify(self);
        [self audoAdjustPlayerViewContainerRatio];
        return YES;
    }];
    
    [self bjl_kvo:BJLMakeProperty(self, currentAlbumIndex) observer:^BJLControlObserving(id  _Nullable value, id  _Nullable oldValue, BJLPropertyChange * _Nullable change) {
        bjl_strongify(self);
        if (self.videoAlbumList.count < 1) { return YES;}
        self.mediaControlView.preButton.enabled = self.currentAlbumIndex > 0;
        self.mediaControlView.nextButton.enabled = self.currentAlbumIndex < (self.videoAlbumList.count - 1);
        return YES;
    }];
}

#pragma mark - action

- (void)video:(BJVPlayInfo *)playInfo playFailedWithError:(NSError *)error {
    NSString *errorTitle;
    switch (error.code) {
        case BJVErrorCode_unknown:
            errorTitle = BJLLocalizedString(@"未知错误");
            break;

        case BJVErrorCode_requestFailed:
            errorTitle = BJLLocalizedString(@"网络请求失败");
            break;

        case BJVErrorCode_invalidToken:
            errorTitle = BJLLocalizedString(@"token 参数错误");
            break;

        case BJVErrorCode_invalidPlayInfo:
            errorTitle = BJLLocalizedString(@"播放信息解析错误");
            break;

        case BJVErrorCode_invalidVideoURL:
            errorTitle = BJLLocalizedString(@"视频 URL 失效");
            break;

        case BJVErrorCode_fileLost:
            errorTitle = BJLLocalizedString(@"播放文件不存在");
            break;

        case BJVErrorCode_playFailed:
            errorTitle = BJLLocalizedString(@"视频播放失败");
            break;

        default:
            break;
    }

    if (errorTitle.length) {
        NSString *detail = [NSString stringWithFormat:@"错误码:%ti \n %@", error.code, [error.userInfo objectForKey:NSLocalizedDescriptionKey] ?: @""];
        [self.reloadView showWithTitle:errorTitle detail:detail];
    }
}

#pragma mark - lamp

// 后台配置或者 api配置的跑马灯有数据则显示，api设置优先级高
- (void)makeLamp {
    if ((self.playerManager.playInfo && self.playerManager.playInfo.lamp.contentMode != BJVLampContentModeNull)
        || self.customLamp.content.length) {
        [self updateLamp];
    }
}

- (void)updateLamp {
    BJVLamp *lamp = self.customLamp ?: self.playerManager.playInfo.lamp;
    [self.playerManager.playInfo.lamp checkContentWithUserName:self.playerManager.userName
                                                    userNumber:self.playerManager.userNumber];
    if (!lamp.content.length || lamp.alpha == 0) {
        return;
    }
    if (!self.lampConstructor) {
        self.lampConstructor = [[BJVLampConstructor alloc] init];
    }
    [self.lampConstructor updateLampWithLamp:lamp
                                    lampView:self.lampView
                                 lampContent:lamp.content];
}

#pragma mark -

- (void)showAuditionTimeStopPlayView {
    [self.playerManager pause];

    BJPUStopPlayAlertView *alertView = [[BJPUStopPlayAlertView alloc] initWithAuditionTimeAlertModel:self.videoOptions.auditionTimeStopPlayModel];
    bjl_weakify(self, alertView);
    alertView.conformCallback = ^() {
        bjl_strongify(self, alertView);
        if (self.auditionAlertCallback) {
            self.auditionAlertCallback(alertView, NO);
        }
    };
    alertView.cancelCallback = ^() {
        bjl_strongify(self, alertView);
        if (self.auditionAlertCallback) {
            self.auditionAlertCallback(alertView, YES);
        }
        else {
            [self cancelAction];
        }
    };

    [self.view addSubview:alertView];
    [alertView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.center.equalTo(self.view);
        make.width.equalTo(@(320));
        make.height.equalTo(@(360));
    }];
}

@end
