//
//  BJPUViewController+protected.h
//  BJVideoPlayerUI
//
//  Created by HuangJie on 2018/3/8.
//

#import <BJVideoPlayerCore/BJVPlayerManager.h>
#import <BJLiveBase/BJLNetworking.h>
#import <BJLiveBase/BJLiveBase+UIKit.h>

#import "BJPUViewController.h"
#import "BJPUMediaControlView.h"
#import "BJPUMediaSettingView.h"
#import "BJPUSliderView.h"
#import "BJPUProgressView.h"
#import "BJPUReloadView.h"
#import "BJPUSubtitleView.h"

#import "BJPUMacro.h"
#import "BJPUTheme.h"
#import "BJPUAppearance.h"
#import "MBProgressHUD+bjp.h"
#import "BJVLampConstructor.h"
#import "BJPUSubTitleLabel.h"
#import "BJVVideoCatalogueItem.h"
#import "BJPUSignInAlertViewController.h"

#if __has_include(<BJVideoPlayerUI/BJVideoPlayerUI-Swift.h>)
    #import <BJVideoPlayerUI/BJVideoPlayerUI-Swift.h>
#elif __has_include("BJVideoPlayerUI-Swift.h")
    #import "BJVideoPlayerUI-Swift.h"
#endif

#if __has_feature(modules) && BJL_USE_SEMANTIC_IMPORT
@import BJPlayerUIBase;
#else
#import <BJPlayerUIBase/BJPlayerUIBase.h>
#endif

NS_ASSUME_NONNULL_BEGIN

@interface BJPUViewController () 

@property (nonatomic, readwrite) BJVPlayerManager *playerManager;
@property (nonatomic) BJPUVideoOptions *videoOptions;
@property (nonatomic) BJLAFNetworkReachabilityManager *reachablityManager;
@property (nonatomic, readwrite) NSTimeInterval currentMaxWatchTime; // 当前最大观看过的时间

@property (nonatomic) BJPUMediaControlView *mediaControlView;
@property (nonatomic) BJPUMediaSettingView *mediaSettingView;
@property (nonatomic, nullable) BJPUSubtitleView *subtitleView;
@property (nonatomic) BJPUSliderView *sliderView;
@property (nonatomic) UIImageView *audioOnlyImageView;
@property (nonatomic) BJPUSubTitleLabel *subtitleLabel;
@property (nonatomic) UIView *topBarView;
@property (nonatomic) UIButton *backButton, *pipButton, *shareButton;
@property (nonatomic) UIButton *lockButton;
@property (nonatomic) BJPUReloadView *reloadView;
@property (nonatomic) UIButton *videoCatalogueButton;
@property (nonatomic) UITableView *videoCatalogueTableView;
@property (nonatomic, nullable) NSArray <BJVVideoCatalogueItem *> *catalogueList;

@property (nonatomic) BOOL fistShowCatalogueViewInHorizen;
@property (nonatomic, nullable) NSString *videoID, *token, *albumNumber;
@property (nonatomic, nullable) BJVDownloadItem *downloadItem;
@property (nonatomic) NSArray *currentOptions;
@property (nonatomic) NSArray *rateList;
@property (nonatomic) NSUInteger rateIndex;
@property (nonatomic) NSUInteger definitionIndex;
@property (nonatomic) BOOL isPlayingTailAD, isLocalVideo, pauseByInterrupt;

@property (nonatomic, readonly) UIView *lampView, *videoAlbumListLayer;
@property (nonatomic) BJVLampConstructor *lampConstructor;

// 只在竖屏状态显示
@property (nonatomic) UIButton *vSubtitleButton;

// 布局容器
@property (nonatomic, strong) UIView *videoContentContainer;
@property (nonatomic, strong) UIView *videoCatalogueContainer;
@property (nonatomic, strong) UIView *tripartiteLayoutRightSideContainer;
@property (nonatomic, strong) UIView *commentVCContainer;

@property (nonatomic, nullable) BJLScreenCaptureAlertMaskView *screenCaptureAlertView;
@property (nonatomic, nullable) BJLScreenRecordingNotifier *screenRecordingNotifier;

@property (nonatomic, nullable) QuizViewController *quizVC;
@property (nonatomic, nullable) TripartiteLayoutOutlinesViewController *outlinesVC;
@property (nonatomic, nullable) CommentListViewController *commentVC;
@property (nonatomic, nullable) BJPUSignInAlertViewController *signInAlertVC;

// 合集
@property (nonatomic, nullable) NSArray<BJVPlayinfoItem *> *videoAlbumList;
@property (nonatomic, assign) NSInteger currentAlbumIndex;
@property (nonatomic, nullable) BJPVideoListViewController *VideoListViewController;
@property (nonatomic) CAGradientLayer *gradientLayer;

- (void)cancelAction;
- (void)setLayoutType:(BJVPlayerViewLayoutType)layoutType shouldupdate:(BOOL)shouldupdate;
- (void)hideView:(UIView *)view withDuration:(NSTimeInterval)duration;
- (void)showSliderDragStopPlayView;
- (void)updateConstriantsByPrefersLayoutTypeIfNeeded;
- (void)playWithVideoInfo:(BJVPlayinfoItem *)item;

- (void)showSubtitleView;
- (void)hideInterfaceViews;
- (void)updateConstriantsWithLayoutType:(BJVPlayerViewLayoutType)layoutType;
- (void)updateWithPlayState:(BJVPlayerStatus)state;
- (void)updateVSubtitleButtonConstriants:(BOOL)subtitleExist;
- (void)audoAdjustPlayerViewContainerRatio;
- (void)setupCatalogueView:(NSArray<BJVVideoCatalogueItem *> *)list;
- (void)updateConstriantsWithTripartiteLayoutInfo:(BJVTripartiteLayoutInfo *)tripartiteLayoutInfo;
- (void)updateConstriantsForCommentsVC:(BOOL)enableVideoComment;
- (void)updatePlayerViewConstraintWithVideoRatio:(CGFloat)ratio;

@end

NS_ASSUME_NONNULL_END
