//
//  BJPUViewController+ui.m
//  BJVideoPlayerUI
//
//  Created by HuangJie on 2018/3/8.
//

#import "BJPUViewController+ui.h"
#import "BJPUViewController+protected.h"

#define kTopBarHeight (40.0)

@implementation BJPUViewController (ui)

- (void)setupSubviews {

    // layout container
    [self.view addSubview:self.videoContentContainer];
    [self.videoContentContainer bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.left.right.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
        make.height.equalTo(self.videoContentContainer.bjl_width).multipliedBy(9.0/16.0);
    }];
    
    // playerView
    UIView *playerView = self.playerManager.playerView;
    [self.videoContentContainer addSubview:playerView];
    [playerView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.edges.equalTo(self.videoContentContainer);
    }];

    // audioOnlyImageView
    [self.videoContentContainer addSubview:self.audioOnlyImageView];
    [self.audioOnlyImageView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.edges.equalTo(playerView);
    }];

    // subtitleLabel
    [self.videoContentContainer addSubview:self.subtitleLabel];
    [self.subtitleLabel bjl_makeConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.centerX.equalTo(self.videoContentContainer);
        make.left.right.equalTo(self.videoContentContainer).inset(10.0).priorityHigh();
        make.bottom.equalTo(self.videoContentContainer).inset(10.0).priorityHigh();
    }];

    // mediaControlView
    [self.videoContentContainer addSubview:self.mediaControlView];
    [self.mediaControlView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.left.right.equalTo(self.videoContentContainer);
        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
            make.bottom.equalTo(self.videoContentContainer);
        }
        else {
            make.bottom.equalTo(playerView);
        }
        make.height.equalTo(@70.0).priorityHigh();
    }];

    // topBarView
    [self.videoContentContainer addSubview:self.topBarView];
    [self updateTopBarViewConstraint];

    // backButton
    UIButton *cancelButton = ({
        UIButton *button = [[UIButton alloc] init];
        button.titleLabel.font = [UIFont systemFontOfSize:16];
        button.titleLabel.lineBreakMode = NSLineBreakByTruncatingTail;
        [button bjl_setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        button.imageView.contentMode = UIViewContentModeScaleAspectFill;
        [button setImage:[BJPUTheme backButtonImage] forState:UIControlStateNormal];
        button.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
        [button addTarget:self action:@selector(backAction) forControlEvents:UIControlEventTouchUpInside];
        button;
    });
    [self.topBarView addSubview:cancelButton];
    [cancelButton bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.top.left.equalTo(self.topBarView);
        make.height.equalTo(@(kTopBarHeight));
        make.right.lessThanOrEqualTo(self.topBarView).offset(-100);
    }];
    self.backButton = cancelButton;
    self.backButton.hidden = !self.showNavigationBackButton;

    if ([self enablePip]) {
        // pipButton
        self.pipButton = [[UIButton alloc] init];
        [self.pipButton setImage:[UIImage bjpu_imageNamed:@"ic_pip_normal"] forState:UIControlStateNormal];
        [self.pipButton addTarget:self action:@selector(pipAction:) forControlEvents:UIControlEventTouchUpInside];
        
        [self.topBarView addSubview:self.pipButton];
        [self.pipButton bjl_makeConstraints:^(BJLConstraintMaker *_Nonnull make) {
            make.centerY.equalTo(self.backButton);
            make.height.width.equalTo(@(30));
            make.right.equalTo(self.topBarView).offset(-5);
        }];
    }
    
    // share
    self.shareButton = [[UIButton alloc] init];
    [self.shareButton setImage:[UIImage bjpu_imageNamed:@"share"] forState:UIControlStateNormal];
    [self.shareButton addTarget:self action:@selector(shareAction) forControlEvents:UIControlEventTouchUpInside];
    self.shareButton.hidden = !self.shareButtonTappedCallback;
    
    [self.topBarView addSubview:self.shareButton];
    [self.shareButton bjl_makeConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.centerY.equalTo(self.backButton);
        make.height.width.equalTo(@(30));
        if (self.pipButton) {
            make.right.equalTo(self.pipButton.bjl_left).offset(-5);
        }
        else {
            make.right.equalTo(self.topBarView).offset(-5);
        }
    }];

    // sliderView
    [self.videoContentContainer addSubview:self.sliderView];
    [self.sliderView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.left.right.equalTo(self.videoContentContainer);
        make.top.equalTo(self.topBarView.bjl_bottom);
        make.bottom.equalTo(self.mediaControlView.bjl_top);
    }];

    // lockButton
    [self.videoContentContainer addSubview:self.lockButton];
    [self.lockButton bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.left.equalTo(self.videoContentContainer).offset(20.0);
        make.centerY.equalTo(self.videoContentContainer);
    }];

    // mediaSettingView
    [self.videoContentContainer addSubview:self.mediaSettingView];
    [self.mediaSettingView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.bottom.equalTo(self.mediaControlView.bjl_top).offset(-2);
        make.width.equalTo(@72.0);
        make.height.equalTo(0);
    }];
    
    [self.videoContentContainer addSubview:self.videoCatalogueContainer];
    [self.videoCatalogueContainer bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.right.bottom.equalTo(self.videoContentContainer);
        make.width.equalTo(@(0));
    }];
    [self makeVideoCatalogueView];
    
    // reload view
    [self.videoContentContainer addSubview:self.videoAlbumListLayer];
    [self.videoAlbumListLayer bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.edges.equalTo(self.videoContentContainer);
    }];
    
    // reload view
    [self.videoContentContainer addSubview:self.reloadView];
    [self.reloadView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.edges.equalTo(self.videoContentContainer);
    }];

    // lamp
    [self.videoContentContainer addSubview:self.lampView];
    [self.lampView bjl_makeConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.edges.equalTo(playerView);
    }];
    
    // quiz
    self.quizVC.parentView = self.view;

    [self setupControlActions];
    [self setupVerticalButton];
    
    if (self.signInList.count > 0) {
        self.signInAlertVC = [[BJPUSignInAlertViewController alloc] initWithPlayer:self.playerManager signInList:self.signInList];
        self.signInAlertVC.parentView = self.view;
        bjl_weakify(self);
        self.signInAlertVC.userDidSignInCallback = ^(BJPUSignInAlertViewController * _Nonnull vc, BJPUSignInModel * _Nonnull signInData) {
            bjl_strongify(self);
            if (self.userDidSignInCallback) {
                self.userDidSignInCallback(self, signInData);
            }
        };
    }
}

- (void)makeAlbumEmptyView {
    UIImageView *bgImageView = [[UIImageView alloc] initWithImage:[UIImage bjpu_imageNamed:@"album_emptybg"]];
    UIImageView *bgPlaceHolderImageView = [[UIImageView alloc] initWithImage:[UIImage bjpu_imageNamed:@"album_emptybg_placeholder"]];
    
    [self.videoContentContainer insertSubview:bgImageView atIndex:0];
    [bgImageView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.center.height.equalTo(self.videoContentContainer);
    }];
    
    UILabel *label = [[UILabel alloc] init];
    label.text = BJLLocalizedString(@"专辑暂无可观看视频");
    label.textColor = [UIColor whiteColor];
    label.font = [UIFont boldSystemFontOfSize:20];
    [self.videoContentContainer addSubview:label];
    [label bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.centerY.equalTo(self.videoContentContainer);
        make.centerX.equalTo(self.videoContentContainer).offset(30);
    }];
    
    [self.videoContentContainer addSubview:bgPlaceHolderImageView];
    [bgPlaceHolderImageView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.centerY.equalTo(label);
        make.right.equalTo(label.bjl_left).offset(-10);
    }];
    
    [self.mediaControlView setControlsHidden:YES];
}

- (void)shareAction {
    if (self.shareButtonTappedCallback) {
        self.shareButtonTappedCallback(self);
    }
}

- (void)updateTopBarViewConstraint {
    CGFloat topBarHeight = kTopBarHeight;
    [self.topBarView bjl_remakeConstraints:^(BJLConstraintMaker *make) {
        if (self.videoContentContainer.bjl_safeAreaLayoutGuide) {
            make.left.top.right.equalTo(self.videoContentContainer.bjl_safeAreaLayoutGuide); //ios11 以上直接用系统api
        }
        else {
            CGFloat statusBarHeight = MAX(20.0, CGRectGetHeight([UIApplication sharedApplication].statusBarFrame));
            if (!self.presentingViewController) {
                statusBarHeight = 0; //这里仅仅present时才会适配状态栏，如果是内嵌view，则不会有顶部偏移
            }
            make.top.equalTo(self.videoContentContainer).offset(statusBarHeight); //ios11一下，没有刘海屏机型，所以适配只用+20pt
            make.left.right.equalTo(self.videoContentContainer);
        }
        make.height.equalTo(@(topBarHeight)).priorityHigh();
    }];
}

- (void)setupVerticalButton {
    self.vSubtitleButton = ({
        UIButton *button = [[UIButton alloc] init];
        button.clipsToBounds = YES;
        [button setTitleColor:[BJPUTheme defaultTextColor] forState:UIControlStateNormal];
        [button.titleLabel setFont:[UIFont systemFontOfSize:14]];
        [button setTitle:BJLLocalizedString(@"字幕") forState:UIControlStateNormal];
        [button addTarget:self action:@selector(vShowSubtitleList) forControlEvents:UIControlEventTouchUpInside];
        button;
    });

    [self.topBarView addSubview:self.vSubtitleButton];
    [self.vSubtitleButton bjl_makeConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.centerY.equalTo(self.topBarView);
        make.height.equalTo(@(30));
        make.right.equalTo(self.pipButton ? self.pipButton.bjl_left : self.topBarView).offset(-5);
        make.width.equalTo(@0.0); // to update
    }];
}

- (void)makeVideoCatalogueView {
    [self.videoContentContainer addSubview:self.videoCatalogueButton];
    [self.videoCatalogueButton bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.equalTo(self.videoContentContainer).offset(70);
        make.height.equalTo(@(44));
        make.width.equalTo(@(24));
        make.right.equalTo(self.videoCatalogueContainer.bjl_left).offset(2);
    }];
    
    bjl_weakify(self);
    [self.videoCatalogueButton bjl_addHandler:^(UIButton * _Nonnull button) {
        bjl_strongify(self);
        [self updateVideoCatalogueButton:NO CatalogueView:self.videoCatalogueButton.selected];
    }];
    
    [self.videoCatalogueContainer addSubview:self.videoCatalogueTableView];
    [self.videoCatalogueTableView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.edges.equalTo(self.videoCatalogueContainer).insets(UIEdgeInsetsMake(10, 8, 8, 10));
    }];
}

- (void)updateVideoCatalogueButton:(BOOL)hidden CatalogueView:(BOOL)catalogueHidden {
    if (self.catalogueList.count <= 0) {
        return;
    }
    
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(hiddenCatalogueView) object:nil];
    
    [UIView animateWithDuration:0.1 delay:0 options:UIViewAnimationOptionCurveLinear animations:^{
        if (catalogueHidden) {
            [self.videoCatalogueContainer bjl_updateConstraints:^(BJLConstraintMaker * _Nonnull make) {
                make.width.equalTo(@(0));
            }];
        }
        else {
            [self.videoCatalogueContainer bjl_updateConstraints:^(BJLConstraintMaker * _Nonnull make) {
                make.width.equalTo(@(190));
            }];
        }
        [self.videoCatalogueContainer layoutIfNeeded];
    } completion:^(BOOL finished) {
        self.videoCatalogueButton.hidden = hidden;
        self.videoCatalogueButton.selected = !catalogueHidden;
    }];
    [self.videoCatalogueTableView reloadData];
    
    if (!catalogueHidden) {
        [self performSelector:@selector(hiddenCatalogueView) withObject:nil afterDelay:5];
    }
}

- (void)hiddenCatalogueView {
    [UIView animateWithDuration:0.1 delay:0 options:UIViewAnimationOptionCurveLinear animations:^{
        [self.videoCatalogueContainer bjl_updateConstraints:^(BJLConstraintMaker * _Nonnull make) {
            make.width.equalTo(@(0));
        }];
        [self.videoCatalogueContainer layoutIfNeeded];
    } completion:^(BOOL finished) {
        self.videoCatalogueButton.selected = NO;
    }];
}

#pragma mark - actions

- (void)setupControlActions {
    // show & hide
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(showOrHideInterfaceViews)];
    [self.sliderView addGestureRecognizer:tap];

    UITapGestureRecognizer *doubleTapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(doubleTapEventHandler)];
    doubleTapGesture.numberOfTapsRequired = 2;
    [self.sliderView addGestureRecognizer:doubleTapGesture];
    [tap requireGestureRecognizerToFail:doubleTapGesture];

    // lock
    [self.lockButton addTarget:self action:@selector(lockAction) forControlEvents:UIControlEventTouchUpInside];

    // cancel
    bjl_weakify(self);
    [self.reloadView setCancelCallback:^{
        bjl_strongify(self);
        [self cancelAction];
    }];
}

- (void)backAction {
    if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad && self.layoutType == BJVPlayerViewLayoutType_Horizon) {
        if (self.playerManager.playInfo.tripartiteLayoutInfo) {
            if (self.screenLockCallback) {
                self.screenLockCallback(NO);
            }
        }

        [self setLayoutType:BJVPlayerViewLayoutType_Vertical];
        
        if (self.playerManager.playInfo.tripartiteLayoutInfo) {
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                [self cancelAction];
            });
        }
    }
    else {
        [self cancelAction];
    }
}

- (void)lockAction {
    BOOL lock = !self.lockButton.selected;
    self.lockButton.selected = lock;
    [self.mediaControlView setControlsHidden:lock];
    self.mediaSettingView.hidden = YES;
    [self updateVideoCatalogueButton:lock CatalogueView:YES];
    self.topBarView.hidden = lock;
    self.sliderView.slideEnabled = !lock;
    [self.mediaControlView setSlideEnable:!lock];
    if (self.screenLockCallback) {
        self.screenLockCallback(lock);
    }
}

- (void)doubleTapEventHandler {
    if (self.lockButton.selected) { return; }

    BJVPlayerStatus status = self.playerManager.playStatus;
    if (status == BJVPlayerStatus_playing) {
        [self.playerManager pause];
    }
    else if (status == BJVPlayerStatus_paused
             || status == BJVPlayerStatus_stopped
             || status == BJVPlayerStatus_reachEnd
             || status == BJVPlayerStatus_failed
             || status == BJVPlayerStatus_ready) {
        [self.playerManager play];
    }
}

- (void)vShowSubtitleList {
    [self showSubtitleView];
}

//- (void)vShowRateList {
//    [self showRateList];
//}

- (void)pipAction:(UIButton *)button {
    if (self.playerManager.pipViewContrller.pictureInPictureActive) {
        [self.pipButton setImage:[UIImage bjpu_imageNamed:@"ic_pip_normal"] forState:UIControlStateNormal];
    }
    else {
        [self.pipButton setImage:[UIImage bjpu_imageNamed:@"ic_pip_active"] forState:UIControlStateNormal];
    }
    
    BOOL shouldActive = !self.playerManager.pipViewContrller.pictureInPictureActive;

    if (shouldActive) {
        [self.playerManager startPictureInPicture];
    }
    else {
        [self.playerManager stopPictureInPicture];
    }
}

- (BOOL)enablePip {
    BOOL enablePip = self.playerManager.playerType == BJVPlayerType_AVPlayer
                     && self.playerManager.pictureInPictureEnabled
                     && [AVPictureInPictureController isPictureInPictureSupported];
    return enablePip;
}

#pragma mark - show & hide

- (void)hideInterfaceViewsAutomatically {
    if (!self.mediaControlView.slideCanceled) {
        // 进度条响应交互中，不执行隐藏
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(hideInterfaceViewsAutomatically) object:nil];
        [self performSelector:@selector(hideInterfaceViewsAutomatically) withObject:nil afterDelay:5.0];
        return;
    }
    [self hideInterfaceViews];
}

- (void)showOrHideInterfaceViews {
    if (self.lockButton.selected) {
        return;
    }
    
    if (self.layoutType == BJVPlayerViewLayoutType_Horizon) {
        self.lockButton.hidden = !self.lockButton.hidden || self.playerManager.playInfo.tripartiteLayoutInfo;
        [self hideView:self.mediaSettingView withDuration:0.5];
    }

    if (!self.mediaSettingView.hidden) {
        self.mediaSettingView.hidden = YES;
    }
    if (self.subtitleView) {
        [self.subtitleView removeFromSuperview];
        self.subtitleView = nil;
    }

    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(hideInterfaceViewsAutomatically) object:nil];
    BOOL hidden = !self.mediaControlView.hidden;
    [self.mediaControlView showOrHideControls];
    self.topBarView.hidden = hidden;
    self.vSubtitleButton.hidden = hidden;
    if (!self.mediaControlView.hidden) {
        [self performSelector:@selector(hideInterfaceViewsAutomatically) withObject:nil afterDelay:5.0];
    }
    if (hidden) {
        [self updateVideoCatalogueButton:(NO || self.layoutType == BJVPlayerViewLayoutType_Vertical)  CatalogueView:YES];
    }
}

- (void)hideMdiaControlView:(BOOL)hidden {
    [self.mediaControlView setControlsHidden:hidden];
    
    [self.subtitleLabel bjl_updateConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.bottom.equalTo(self.videoContentContainer.bjl_safeAreaLayoutGuide ?: self.videoContentContainer).inset(hidden ? 10.0 : 40.0).priorityHigh();    }];
}

#pragma mark - public

- (void)_setupCatalogueView:(NSArray<BJVVideoCatalogueItem *> *)list {
    self.catalogueList = list;
    if (list.count <= 0) {
        self.videoCatalogueButton.hidden = YES;
        return;
    }
    
    // 处理首次横屏之后 打点还没有加载出来的情况
    BOOL horizen = self.layoutType == BJVPlayerViewLayoutType_Horizon;
    if (self.videoCatalogueButton.hidden && horizen && !self.fistShowCatalogueViewInHorizen) {
        self.fistShowCatalogueViewInHorizen = YES;
        [self updateVideoCatalogueButton:NO CatalogueView:NO];
    }

    [self.videoCatalogueTableView reloadData];
}

- (void)_updateConstriantsWithTripartiteLayoutInfo:(BJVTripartiteLayoutInfo *)tripartiteLayoutInfo {
    [self.mediaControlView setScaleEnable:!tripartiteLayoutInfo];
    self.lockButton.hidden = (tripartiteLayoutInfo != nil);
    if (self.screenLockCallback) {
        self.screenLockCallback(tripartiteLayoutInfo != nil);
    } 
    if (tripartiteLayoutInfo == nil) {
        if (self.outlinesVC) {
            [self.outlinesVC bjl_removeFromParentViewControllerAndSuperiew];
            self.outlinesVC = nil;
        }
        
        [self.tripartiteLayoutRightSideContainer removeFromSuperview];
        
        [self.videoContentContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
            make.edges.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
        }];
        return;
    }
    
    if (self.outlinesVC) {
        [self.outlinesVC bjl_removeFromParentViewControllerAndSuperiew];
        self.outlinesVC = nil;
    }
    
    [self.view addSubview:self.tripartiteLayoutRightSideContainer];
    [self.tripartiteLayoutRightSideContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.right.bottom.equalTo(self.view);
        make.width.equalTo(self.view).multipliedBy(0.22);
    }];
    [self.videoContentContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.left.bottom.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
        make.right.equalTo(self.tripartiteLayoutRightSideContainer.bjl_left);
    }];
    [self.mediaControlView bjl_remakeConstraints:^(BJLConstraintMaker *make) {
        make.left.right.equalTo(self.videoContentContainer);
        make.bottom.equalTo(self.videoContentContainer);
        make.height.equalTo(@70.0).priorityHigh();
    }];

    self.outlinesVC = [[TripartiteLayoutOutlinesViewController alloc] initWithPlayerManager:self.playerManager outlines:tripartiteLayoutInfo];
    bjl_weakify(self);
    self.outlinesVC.swapViewEventHandler = ^UIView * _Nullable(UIView * _Nonnull view, BOOL didSwap) {
        bjl_strongify(self);
        if (!didSwap) {//  ppt为主
            [self.videoContentContainer insertSubview:view atIndex:0];
            [view bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
                make.edges.equalTo(self.videoContentContainer);
            }];
            
            [self.audioOnlyImageView removeFromSuperview];
            [self.playerManager.playerView addSubview:self.audioOnlyImageView];
            [self.audioOnlyImageView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
                make.edges.equalTo(self.playerManager.playerView);
            }];
            [self.lampView removeFromSuperview];
            [self.playerManager.playerView addSubview:self.lampView];
            [self.lampView bjl_remakeConstraints:^(BJLConstraintMaker *_Nonnull make) {
                make.edges.equalTo(self.playerManager.playerView);
            }];

            return self.playerManager.playerView;
        }
        else {//视频为主
            [self.videoContentContainer insertSubview:self.playerManager.playerView atIndex:0];
            [self _audoAdjustPlayerViewContainerRatio];
            [self.audioOnlyImageView removeFromSuperview];
            [self.videoContentContainer insertSubview:self.audioOnlyImageView aboveSubview:self.playerManager.playerView];
            [self.audioOnlyImageView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
                make.edges.equalTo(self.videoContentContainer);
            }];

            [self.lampView removeFromSuperview];
            [self.playerManager.playerView addSubview:self.lampView];
            [self.lampView bjl_remakeConstraints:^(BJLConstraintMaker *_Nonnull make) {
                make.edges.equalTo(self.playerManager.playerView);
            }];
            return nil;
        }
    };
    [self bjl_addChildViewController:self.outlinesVC superview:self.tripartiteLayoutRightSideContainer];
    [self.outlinesVC.view bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.edges.equalTo(self.tripartiteLayoutRightSideContainer);
    }];
    [self updateConstriantsWithLayoutType:BJVPlayerViewLayoutType_Horizon];
}

- (void)_updateConstriantsForCommentsVC:(BOOL)enableVideoComment {
    if (!enableVideoComment || self.videoID.length == 0) {
        [self.videoContentContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
            make.edges.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
        }];
        [self.commentVCContainer removeFromSuperview];
        return;
    }
    
    [self.videoContentContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.edges.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
    }];
    [self.view insertSubview:self.commentVCContainer belowSubview:self.videoContentContainer];
    [self.commentVCContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.equalTo(self.videoContentContainer.bjl_bottom);
        make.left.right.bottom.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
    }];
    self.commentVC = [[CommentListViewController alloc] initWithVideoID:self.videoID];
    [self bjl_addChildViewController:self.commentVC superview:self.commentVCContainer];
    [self.commentVC.view bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.edges.equalTo(self.commentVCContainer);
    }];
    [self updateConstriantsWithLayoutType:BJVPlayerViewLayoutType_Vertical];
}

- (void)_updateConstriantsWithLayoutType:(BJVPlayerViewLayoutType)layoutType {
    BOOL horizen = (layoutType == BJVPlayerViewLayoutType_Horizon);
    BOOL locked = self.lockButton.selected;
    if (!self.lockButton.hidden && locked && !horizen) {
        [self setLayoutType:BJVPlayerViewLayoutType_Horizon];
        return;
    }
    [self setLayoutType:layoutType shouldupdate:NO];
    
    if (!self.playerManager.playInfo.tripartiteLayoutInfo
        && self.playerManager.playInfo.enableVideoComment) {
        // 带评论的垂直方向居顶
        [self.videoContentContainer bjl_remakeConstraints:^(BJLConstraintMaker * _Nonnull make) {
            if (!horizen) {
                make.top.left.right.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
                make.height.equalTo(self.videoContentContainer.bjl_width).multipliedBy(9.0/16.0);
            }
            else {
                make.edges.equalTo(self.view.bjl_safeAreaLayoutGuide ?: self.view);
            }
        }];
    }
    
    [self.mediaControlView updateConstraintsWithLayoutType:horizen];
    BOOL controlHidden = self.mediaControlView.hidden;
    
    // mediaSettingView 和 subtitleView, 维持原来的状态
    self.mediaSettingView.hidden = self.mediaSettingView.hidden;
    self.subtitleView.hidden = self.subtitleView.hidden;
    
    // lockButton只有在横屏下才显示
    self.lockButton.hidden = !horizen || controlHidden || self.playerManager.playInfo.tripartiteLayoutInfo;
        
    [self.vSubtitleButton bjl_updateConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.width.equalTo((self.mediaControlView.existSubtitle && !horizen) ? @45.0 : @0.0);
    }];
    
    [self updateTopBarViewConstraint];
    
    BOOL hiddenCatalogueView = YES;
    if (self.catalogueList.count > 0 && horizen && !self.fistShowCatalogueViewInHorizen) {
        hiddenCatalogueView = NO;
        self.fistShowCatalogueViewInHorizen = YES;
    }
    [self updateVideoCatalogueButton:!horizen CatalogueView:hiddenCatalogueView];
    
    if (self.commentVC) {
        self.commentVCContainer.hidden = horizen;
    }
    
    self.view.backgroundColor = horizen ? UIColor.blackColor : UIColor.whiteColor;
}

- (void)_audoAdjustPlayerViewContainerRatio {
    // 视频宽高比
    BJVDefinitionInfo *definitionInfo = self.playerManager.currDefinitionInfo;
    CGFloat width = definitionInfo.width;
    CGFloat height = definitionInfo.height;
    if (width > 0.0 && height > 0.0) {
        CGFloat videoRatio = width / height;
        // 更新播放视图宽高比
        [self updatePlayerViewConstraintWithVideoRatio:videoRatio];
    }
}

- (void)_updatePlayerViewConstraintWithVideoRatio:(CGFloat)ratio {
    UIView *playerView = self.playerManager.playerView;
    if (!playerView.superview) { return; }
    
    [playerView bjl_remakeConstraints:^(BJLConstraintMaker *make) {
        if (ratio > 0) {
            
            BOOL isFit = self.videoOptions.videoFit;
            
            // 15以上系统， size超出 会不显示画面，只播放声音
            CGFloat superViewRatio = 16.0 / 9.0;
            if (playerView.superview.frame.size.width > 0.0 &&
                playerView.superview.frame.size.height > 0.0) {
                CGFloat width = playerView.superview.frame.size.width;
                CGFloat height = playerView.superview.frame.size.height;
                superViewRatio = width / height;
            }
            make.height.equalTo(playerView.bjl_width).dividedBy(ratio);
            if (isFit) {
                if (ratio >= superViewRatio) {
                    make.center.width.equalTo(playerView.superview);
                }
                else {
                    make.center.height.equalTo(playerView.superview);
                }
            }
            else {
                if (ratio >= superViewRatio) {
                    make.center.height.equalTo(playerView.superview);
                }
                else {
                    make.center.width.equalTo(playerView.superview);
                }
            }
        }
        else {
            make.edges.equalTo(self.videoContentContainer);
        }
    }];
}

- (void)_updateWithPlayState:(BJVPlayerStatus)state {
    if (state == BJVPlayerStatus_paused || state == BJVPlayerStatus_stopped || state == BJVPlayerStatus_reachEnd || state == BJVPlayerStatus_failed || state == BJVPlayerStatus_ready) {
        [self.mediaControlView updateWithPlayState:NO];
    }
    else if (state == BJVPlayerStatus_playing) {
        [self.mediaControlView updateWithPlayState:YES];
    }
}

- (void)_updateVSubtitleButtonConstriants:(BOOL)subtitleExist {
    [self.vSubtitleButton bjl_updateConstraints:^(BJLConstraintMaker *_Nonnull make) {
        // 不是横屏 && 字幕存在的时候, 显示 vSubtitleButton
        make.width.equalTo((!BJPUIsHorizontalUI(self) && subtitleExist) ? @45.0 : @0.0);
    }];
}
@end
