//
//  BJPUViewController.m
//  BJVideoPlayerUI
//
//  Created by DLM on 2017/4/26.
//
//

#import "BJPUViewController.h"
#import "BJPUViewController+protected.h"
#import "BJPUViewController+media.h"
#import "BJPUViewController+ui.h"
#import "BJPUViewController+observer.h"
#import "BJPUViewController+reachability.h"
#import <BJPlayerUIBase/BJPUStopPlayAlertView.h>

@interface BJPUViewController () <BJPUSliderProtocol, UITableViewDelegate, UITableViewDataSource>

@property (nonatomic, assign) CGFloat seekTargetTime;

@end

@implementation BJPUViewController

- (instancetype)initWithVideoOptions:(BJPUVideoOptions *)videoOptions {
    self = [super init];
    if (self) {
        self.showNavigationBackButton = YES;
        self.autoAdjustDeviceOrientationByFullScreenState = YES;
        [self updateVideoOptions:videoOptions];
    }
    return self;
}

- (void)updateVideoOptions:(BJPUVideoOptions *)videoOptions {
    self.videoOptions = videoOptions;
    self.playerManager.backgroundAudioEnabled = videoOptions.backgroundAudioEnabled;
    self.playerManager.remoteControlEnabled = videoOptions.remoteControlEnabled;
    self.playerManager.remoteControlImage = videoOptions.remoteControlImage;
    self.playerManager.pictureInPictureEnabled = videoOptions.pictureInPictureEnabled;
    self.playerManager.preferredDefinitionList = videoOptions.preferredDefinitionList;
    self.playerManager.playTimeRecordEnabled = videoOptions.playTimeRecordEnabled;
    self.playerManager.initialPlayTime = videoOptions.initialPlayTime;
    self.playerManager.waterMarkModel = videoOptions.waterMarkModel;
    self.playerManager.userName = videoOptions.userName;
    self.playerManager.userNumber = videoOptions.userNumber;
}

- (void)dealloc {
    [_screenRecordingNotifier stopObserve];
    [self.reachablityManager stopMonitoring];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self.playerManager destroy];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
    // 子视图
    [self setupSubviews];
    // 监听
    [self setupObservers];
    // callback
    [self setMediaCallbacks];
    // 网络检查
    [self setupReachabilityManager];
    
    [NSNotificationCenter.defaultCenter addObserver:self selector:@selector(deviceOrientationChanged) name:UIDeviceOrientationDidChangeNotification object:nil];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // 更新布局
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [NSObject cancelPreviousPerformRequestsWithTarget:self];
}

//- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
//    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
//    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext> _Nonnull context) {
//        [self setNeedsStatusBarAppearanceUpdate];
//        [self.view setNeedsUpdateConstraints];
//        BOOL isHorizontal = BJPUIsHorizontalUI(self);
//        [self updateConstriantsWithLayoutType:isHorizontal ? BJVPlayerViewLayoutType_Horizon : BJVPlayerViewLayoutType_Vertical];
//    } completion:nil];
//}
//
//- (void)willTransitionToTraitCollection:(UITraitCollection *)newCollection withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
//    [super willTransitionToTraitCollection:newCollection withTransitionCoordinator:coordinator];
//    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext> _Nonnull context) {
//        [self setNeedsStatusBarAppearanceUpdate];
//        if (@available(iOS 11.0, *)) {
//            [self setNeedsUpdateOfHomeIndicatorAutoHidden];
//        }
//        [self.view setNeedsUpdateConstraints];
//    } completion:nil];
//}

- (void)deviceOrientationChanged {
    if (self.playerManager.playInfo.tripartiteLayoutInfo || !self.autoAdjustDeviceOrientationByFullScreenState) {
        return;
    }
    [self setNeedsStatusBarAppearanceUpdate];
    if (@available(iOS 11.0, *)) {
        [self setNeedsUpdateOfHomeIndicatorAutoHidden];
    }
    
    UIDeviceOrientation orientation = UIDevice.currentDevice.orientation;
    if (orientation == UIDeviceOrientationLandscapeLeft || orientation == UIDeviceOrientationLandscapeRight) {
        [self updateConstriantsWithLayoutType: BJVPlayerViewLayoutType_Horizon];
    }
    else if (orientation == UIDeviceOrientationPortrait || orientation == UIDeviceOrientationPortraitUpsideDown) {
        [self updateConstriantsWithLayoutType: BJVPlayerViewLayoutType_Vertical];
    }
}

- (BOOL)prefersHomeIndicatorAutoHidden {
    BOOL isHorizontal = BJPUIsHorizontalUI(self);
    return isHorizontal;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    if (self.supportedOrientations == 0) {
        return [super supportedInterfaceOrientations];
    }
    else {
        return self.supportedOrientations;
    }
}

- (void)setShowNavigationBackButton:(BOOL)showNavigationBackButton {
    _showNavigationBackButton = showNavigationBackButton;
    self.backButton.hidden = !showNavigationBackButton;
}

#pragma mark - setters & getters

- (BJVPlayerManager *)playerManager {
    if (!_playerManager) {
        _playerManager = ({
            BJVPlayerManager *manager = [[BJVPlayerManager alloc] initWithPlayerType:self.videoOptions.playerType];
            manager;
        });
    }
    return _playerManager;
}

@synthesize lampView = _lampView;
- (UIView *)lampView {
    if (!_lampView) {
        _lampView = [BJLHitTestView new];
        _lampView.accessibilityIdentifier = BJLKeypath(self, lampView);
        _lampView.clipsToBounds = YES;
    }
    return _lampView;
}

@synthesize videoAlbumListLayer = _videoAlbumListLayer;
- (UIView *)videoAlbumListLayer {
    return _videoAlbumListLayer ?: (_videoAlbumListLayer = ({
        UIView *view = [UIView new];
        view.accessibilityIdentifier = BJLKeypath(self, videoAlbumListLayer);
        
        bjl_weakify(view);
        UITapGestureRecognizer *tapGesture = [UITapGestureRecognizer bjl_gestureWithHandler:^(__kindof UIGestureRecognizer *_Nullable gesture) {
            bjl_strongify(view);
            view.hidden = YES;
        }];
        [view addGestureRecognizer:tapGesture];

        UIPanGestureRecognizer *panGesture = [UIPanGestureRecognizer bjl_gestureWithHandler:^(__kindof UIGestureRecognizer *_Nullable gesture) {
            bjl_strongify(view);
            view.hidden = YES;
        }];
        [view addGestureRecognizer:panGesture];
        view.hidden = YES;
        view;
    }));
}

- (BJPUSubTitleLabel *)subtitleLabel {
    if (!_subtitleLabel) {
        _subtitleLabel = ({
            BJPUSubTitleLabel *label = [BJPUSubTitleLabel new];
            label.numberOfLines = 0;
            label.backgroundColor = [UIColor clearColor];
            label.accessibilityIdentifier = BJLKeypath(self, subtitleLabel);
            label.textColor = [BJPUTheme defaultTextColor];
            label.font = [UIFont systemFontOfSize:16.0 weight:UIFontWeightMedium];
            label.textAlignment = NSTextAlignmentCenter;
            label.shadowColor = [UIColor colorWithWhite:0.0 alpha:1];
            label.shadowOffset = CGSizeMake(-1.0, 1.0);
            label.layer.masksToBounds = false;
            label.layer.shadowRadius = 2.0;
            label;
        });
    }
    return _subtitleLabel;
}

- (BJPUMediaControlView *)mediaControlView {
    if (!_mediaControlView) {
        _mediaControlView = ({
            BJPUMediaControlView *view = [[BJPUMediaControlView alloc] initWithPlayerManager:self.playerManager];
            [view setSlideEnable:self.videoOptions.sliderDragEnabled];
            [view setVideoRateEnable:self.videoOptions.enableVideoRate];
            [view setExternalAlnumShow:self.videoOptions.externalAlnumShow];
            view;
        });
    }
    return _mediaControlView;
}

- (BJPUMediaSettingView *)mediaSettingView {
    if (!_mediaSettingView) {
        _mediaSettingView = [[BJPUMediaSettingView alloc] init];
        _mediaSettingView.hidden = YES;
        _mediaSettingView.layer.cornerRadius = 4;
        _mediaSettingView.layer.masksToBounds = YES;
    }
    return _mediaSettingView;
}

- (BJPUSliderView *)sliderView {
    if (!_sliderView) {
        _sliderView = ({
            BJPUSliderView *sliderView = [[BJPUSliderView alloc] init];
            sliderView.delegate = self;
            sliderView.slideEnabled = self.videoOptions.sliderDragEnabled;
            sliderView;
        });
    }
    return _sliderView;
}

- (UIImageView *)audioOnlyImageView {
    if (!_audioOnlyImageView) {
        _audioOnlyImageView = ({
            UIImageView *imageView = [[UIImageView alloc] init];
            imageView.image = [UIImage bjpu_imageNamed:@"ic_audio_only"];
            imageView;
        });
    }
    return _audioOnlyImageView;
}

- (UIView *)topBarView {
    if (!_topBarView) {
        _topBarView = ({
            UIView *view = [[UIView alloc] init];
            view.backgroundColor = [[BJPUTheme brandColor] colorWithAlphaComponent:0.3];
            view;
        });
    }
    return _topBarView;
}

- (UIButton *)lockButton {
    if (!_lockButton) {
        _lockButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button setImage:[UIImage bjpu_imageNamed:@"ic_lock"] forState:UIControlStateNormal];
            [button setImage:[UIImage bjpu_imageNamed:@"ic_unlock"] forState:UIControlStateSelected];
            button;
        });
    }
    return _lockButton;
}

- (BJPUReloadView *)reloadView {
    if (!_reloadView) {
        _reloadView = ({
            BJPUReloadView *view = [[BJPUReloadView alloc] init];
            view.hidden = YES;
            view;
        });
    }
    return _reloadView;
}

- (UIButton *)videoCatalogueButton {
    if (!_videoCatalogueButton) {
        _videoCatalogueButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button setImage:[UIImage bjpu_imageNamed:@"videoCatalogue_close"] forState:UIControlStateNormal];
            [button setImage:[UIImage bjpu_imageNamed:@"videoCatalogue_open"] forState:UIControlStateSelected];
            [button setBackgroundColor:[UIColor bjl_colorWithHex:0x313847 alpha:0.8]];
            button.layer.cornerRadius = 4.0;
            button.layer.masksToBounds = YES;
            button.hidden = YES;
            button;
        });
    }
    return _videoCatalogueButton;
}

- (UITableView *)videoCatalogueTableView {
    if (!_videoCatalogueTableView) {
        _videoCatalogueTableView = [UITableView new];
        _videoCatalogueTableView.delegate = self;
        _videoCatalogueTableView.dataSource = self;
        _videoCatalogueTableView.backgroundColor = UIColor.clearColor;
        [_videoCatalogueTableView registerClass:[BJPUVideoCatalogueCell class] forCellReuseIdentifier:NSStringFromClass([BJPUVideoCatalogueCell class])];
        [_videoCatalogueTableView registerClass:[UITableViewCell class] forCellReuseIdentifier:NSStringFromClass([UITableViewCell class])];
    }
    return _videoCatalogueTableView;
}

- (NSArray *)rateList {
    if (!_rateList) {
        // iOS 10以下系统在 0.7、1.2 倍速时会出现音视频不同步，换成 0.8、1.25
        _rateList = @[@0.75, @1.0, @1.25, @1.5, @1.75, @2.0];
    }
    return _rateList;
}

- (UIView *)videoContentContainer {
    if (!_videoContentContainer) {
        _videoContentContainer = [[UIView alloc] init];
        _videoContentContainer.backgroundColor = UIColor.blackColor;
        _videoContentContainer.clipsToBounds = YES;
    }
    return _videoContentContainer;
}

- (UIView *)videoCatalogueContainer {
    if (!_videoCatalogueContainer) {
        _videoCatalogueContainer = [[UIView alloc] init];
        _videoCatalogueContainer.backgroundColor = [UIColor bjl_colorWithHex:0x313847 alpha:0.8];
    }
    return _videoCatalogueContainer;
}

- (UIView *)tripartiteLayoutRightSideContainer {
    if (!_tripartiteLayoutRightSideContainer) {
        _tripartiteLayoutRightSideContainer = [[UIView alloc] init];
        _tripartiteLayoutRightSideContainer.backgroundColor = UIColor.clearColor;
    }
    return _tripartiteLayoutRightSideContainer;
}

- (UIView *)commentVCContainer {
    if(!_commentVCContainer) {
        _commentVCContainer = [[UIView alloc] init];
        _commentVCContainer.backgroundColor = [UIColor clearColor];
        _commentVCContainer.accessibilityIdentifier = @"_commentVCContainer";
    }
    return _commentVCContainer;
}

- (CAGradientLayer *)gradientLayer {
    return _gradientLayer ?: (_gradientLayer = ({
        CAGradientLayer *gl = [CAGradientLayer layer];
        gl.startPoint = CGPointMake(0.17, 0.50);
        gl.endPoint = CGPointMake(0.68, 0.50);
        gl.colors = @[
        (__bridge id)[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.00].CGColor,
        (__bridge id)[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.6].CGColor,
        ];
        gl.locations = @[@(0), @(1)];
        gl;
    }));
}

#pragma mark - public interface

- (void)playWithVid:(NSString *)vid token:(NSString *)token {
    self.isLocalVideo = NO;
    self.videoID = vid;
    self.token = token;
    self.downloadItem = nil;

    [self clearOverlayViews];
    [self.playerManager setupOnlineVideoWithID:vid
                                         token:token
                                     encrypted:self.videoOptions.encryptEnabled
                                     accessKey:nil];
    [self.quizVC updateWithVid:vid
                         token:token
                      userName:self.quizUserName ?: @"PlaybackStudent"
                    userNumber:self.quizUserNumber ?: @"10086" ];
    
    if (self.videoOptions.autoplay) {
        [self.playerManager play];
    }
}

- (void)playWithDownloadItem:(BJVDownloadItem *)downloadItem {
    self.isLocalVideo = YES;
    self.downloadItem = downloadItem;
    self.videoID = nil;
    self.token = nil;

    // 清理 overlay
    [self clearOverlayViews];

    // 初始化播放器
    [self.playerManager setupLocalVideoWithDownloadItem:downloadItem];
    if (self.videoOptions.autoplay) {
        [self.playerManager play];
    }
}

- (void)playWithVideoAlbumNumber:(NSString *)number {
    self.albumNumber = number;
    self.videoID = nil;
    self.token = nil;
    self.isLocalVideo = NO;
    self.downloadItem = nil;
    
    [self clearOverlayViews];
    // 通过专辑获取列表，拿列表第一个获取token
    
    bjl_weakify(self);
    [BJLProgressHUD bjp_showLoading:nil toView:self.view];
    [self.playerManager getVideoAlbumListWithNumber:number completion:^(NSURLSessionDataTask * _Nullable task, NSArray<BJVPlayinfoItem *> * _Nullable list, NSError * _Nullable error) {
        bjl_strongify(self);
        self.videoAlbumList = list;
        [self.mediaControlView setSwitchVideoEnable:self.videoAlbumList.count > 0];
        BJVPlayinfoItem *first = self.videoAlbumList.firstObject;
        self.currentAlbumIndex = 0;
        if (!first) {
            [self makeAlbumEmptyView];
            [BJLProgressHUD bjp_closeLoadingView:self.view];
            return;
        }
        [self playWithVideoInfo:first];
    }];
}

- (void)setLayoutType:(BJVPlayerViewLayoutType)layoutType {
    BOOL shouldupdate = (_layoutType != layoutType);
    [self setLayoutType:layoutType shouldupdate:shouldupdate];
}

- (void)setLayoutType:(BJVPlayerViewLayoutType)layoutType shouldupdate:(BOOL)shouldupdate {
    _layoutType = layoutType;
    
    [self bjl_setNeedsUpdateOfSupportedInterfaceOrientations];
    
    if (self.autoAdjustDeviceOrientationByFullScreenState) {
        BOOL horizon = BJPUIsHorizontalUI(self);
        if (layoutType == BJVPlayerViewLayoutType_Horizon && !horizon) {
            if (@available(iOS 16.0, *)) {
                UIInterfaceOrientationMask orientation = UIInterfaceOrientationMaskLandscapeLeft | UIInterfaceOrientationMaskLandscapeRight;
                [UIDevice bjl_updateDeviceOrientationMask:orientation errorHandler:nil];
            } else {
                [[UIDevice currentDevice] setValue:@(UIDeviceOrientationLandscapeLeft) forKey:@"orientation"];
            }
        }
        else if (layoutType == BJVPlayerViewLayoutType_Vertical && horizon) {
            if (@available(iOS 16.0, *)) {
                [UIDevice bjl_updateDeviceOrientationMask:UIInterfaceOrientationMaskPortrait errorHandler:nil];
            } else {
                [[UIDevice currentDevice] setValue:@(UIDeviceOrientationPortrait) forKey:@"orientation"];
            }
        }
    }
    
    if (shouldupdate) {
        [self updateConstriantsWithLayoutType:_layoutType];
    }
}

- (void)updateConstriantsByPrefersLayoutTypeIfNeeded {
    if (!self.viewLoaded){ return; }
    
    BOOL ignorePrefersLayoutType = NO;
    if (!self.playerManager.playInfo.tripartiteLayoutInfo && self.playerManager.playInfo.enableVideoComment) {
        ignorePrefersLayoutType = YES;
    }
    else if (self.playerManager.playInfo.tripartiteLayoutInfo) {
        ignorePrefersLayoutType = YES;
    }
    if (ignorePrefersLayoutType) { return; }
    
    BJVPlayerViewLayoutType layout = self.videoOptions.isLandscapeLayout ? BJVPlayerViewLayoutType_Horizon : BJVPlayerViewLayoutType_Vertical;
    [self setLayoutType:layout shouldupdate:YES];
}

- (void)showSliderDragStopPlayView {
    BJPUStopPlayAlertView *alertView = [[BJPUStopPlayAlertView alloc] initWithSliderDragAlertModel:self.videoOptions.sliderDragStopPlayModel];
    bjl_weakify(alertView);
    alertView.conformCallback = ^() {
        bjl_strongify(alertView);
        [alertView removeFromSuperview];
        alertView = nil;
    };
    
    [self.view addSubview:alertView];
    [alertView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.center.equalTo(self.view);
        make.width.equalTo(@(320));
        make.height.equalTo(@(220));
    }];
}

#pragma mark - BJPUSliderProtocol

- (CGFloat)originValueForTouchSlideView:(BJPUSliderView *)touchSlideView {
    return self.playerManager.currentTime;
}

- (CGFloat)durationValueForTouchSlideView:(BJPUSliderView *)touchSlideView {
    return self.playerManager.duration;
}

- (void)touchSlideView:(BJPUSliderView *)touchSlideView finishHorizonalSlide:(CGFloat)value {
    if (!self.videoOptions.sliderAdvanceEnabled && value > self.currentMaxWatchTime) {
        [self showSliderDragStopPlayView];
        return;
    }

    // 控制频率，延时执行
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(videoSeekAction) object:nil];
    self.seekTargetTime = value;
    [self performSelector:@selector(videoSeekAction) withObject:nil afterDelay:0.8];
}

#pragma mark - UITableViewDelegate, UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [self.catalogueList count];
}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    BJVVideoCatalogueItem *item = [self.catalogueList bjl_objectAtIndex:indexPath.section];
    if (!item) {
        return [UITableViewCell new];
    }
    BJPUVideoCatalogueCell *cell = [tableView dequeueReusableCellWithIdentifier:NSStringFromClass([BJPUVideoCatalogueCell class])];
    [cell updateWithModel:item];
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    BJVVideoCatalogueItem *item = [self.catalogueList bjl_objectAtIndex:indexPath.section];
    if (!item) {
        return ;
    }

    BJPUVideoCatalogueCell *cell = [tableView dequeueReusableCellWithIdentifier:NSStringFromClass([BJPUVideoCatalogueCell class])];
    cell.selected = YES;
    
    NSString *timeStr = item.timeOffset;
    NSArray *timeArr = [timeStr componentsSeparatedByString:@":"];
    NSInteger hour = 0;
    NSInteger mins = 0;
    NSInteger second = 0;

    if (timeArr.count == 3) {
        second = [timeArr[2] integerValue];
        mins = [timeArr[1] integerValue];
        hour = [timeArr[0] integerValue];
    }
    else if (timeArr.count == 2) {
        second = [timeArr[1] integerValue];
        mins = [timeArr[0] integerValue];
    }
    else if (timeArr.count == 1) {
        second = [timeArr[0] integerValue];
    }
    NSInteger seconds = hour * 3600 + mins * 60 + second;

    [self.playerManager seek:seconds];
}

- (BOOL)tableView:(UITableView *)tableView shouldHighlightRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section {
    return [UIView new];
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    return 6;
}

#pragma mark - action
- (void)hideView:(UIView *)view withDuration:(NSTimeInterval)duration {
    [UIView animateWithDuration:0.5 animations:^{
        view.alpha = 0.0;
    } completion:^(BOOL finished) {
        view.hidden = YES;
        view.alpha = 1.0;
    }];
}

- (void)videoSeekAction {
    [self seekToTime:self.seekTargetTime];
}

- (void)clearOverlayViews {
    self.reloadView.hidden = YES;
    self.mediaSettingView.hidden = YES;
}

- (void)cancelAction {
    if (self.cancelCallback) {
        self.cancelCallback();
    }
    else if (self.navigationController) {
        [self.navigationController popViewControllerAnimated:YES];
    }
    else if (self.presentingViewController) {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
    [self.playerManager destroy];
}

- (nullable BJLScreenCaptureAlertMaskView *)screenCaptureAlertView {
    if (!_screenCaptureAlertView) {
        _screenCaptureAlertView = [[BJLScreenCaptureAlertMaskView alloc] init];
        bjl_weakify(self);
        _screenCaptureAlertView.dismissEventHandler = ^(BJLScreenCaptureAlertMaskView *_Nonnull view) {
            bjl_strongify(self);
            [self cancelAction];
        };
    }
    return _screenCaptureAlertView;
}

- (BJLScreenRecordingNotifier *)screenRecordingNotifier {
    if (!_screenRecordingNotifier) {
        _screenRecordingNotifier = [[BJLScreenRecordingNotifier alloc] init];
        bjl_weakify(self);
        _screenRecordingNotifier.screenRecordingStateChangeBlock = ^(BJLScreenRecordingNotifier *_Nonnull notifier, BOOL screenRecording) {
            bjl_strongify(self);

            if (!self.playerManager.playInfo.enablePreventScreenCapture) {
                return;
            }

            if (screenRecording) {
                [self.screenCaptureAlertView showInParentView:self.view];
            }
            else {
                [self.screenCaptureAlertView hide];
            }
        };
    }
    return _screenRecordingNotifier;
}

- (QuizViewController *)quizVC {
    if(!_quizVC) {
        _quizVC = [[QuizViewController alloc] initWithPlayerManager:self.playerManager];
    }
    return _quizVC;
}

- (void)setShareButtonTappedCallback:(void (^)(BJPUViewController * _Nonnull))shareButtonTappedCallback {
    _shareButtonTappedCallback = shareButtonTappedCallback;
    self.shareButton.hidden = !shareButtonTappedCallback;
}

- (void)playWithVideoInfo:(BJVPlayinfoItem *)item {
    bjl_weakify(self);
    [self.playerManager reset];
    [self.VideoListViewController updateCuttentPlayInfoWithInfo:item];
    [self.playerManager getVideoAlbumTokenWithNumber:self.albumNumber vid:item.videoID completion:^(NSURLSessionDataTask * _Nullable task, NSString * vid, NSString * _Nullable token, NSError * _Nullable error) {
        bjl_strongify(self);
        [BJLProgressHUD bjp_closeLoadingView:self.view];
        if (!token || token.length <= 0) {
            [BJLProgressHUD bjp_showMessageThenHide:@"合集播放token为空" toView:self.view onHide:nil];
            return;
        }
        [self playWithVid:vid token:token];
    }];
}

- (void)showSubtitleView {
    NSArray<BJVSubtitleInfo *> *infos = self.playerManager.playInfo.subtitleInfo;
    NSMutableArray<NSString *> *infoName = [NSMutableArray new];
    NSInteger selectedIndex = 0;
    for (NSInteger index = 0; index < infos.count; index++) {
        BJVSubtitleInfo *info = [infos bjl_objectAtIndex:index];
        if (info.ID == self.playerManager.currentSubtitleInfo.ID) {
            selectedIndex = index;
        }
        [infoName bjl_addObject:info.name];
    }

    [self hideInterfaceViews];
    self.subtitleView = [BJPUSubtitleView new];
    [self.subtitleView updateWithSettingOptons:infoName selectedIndex:selectedIndex on:!self.subtitleLabel.hidden];
    bjl_weakify(self);
    [self.subtitleView setSelectCallback:^(NSUInteger selectedIndex) {
        bjl_strongify(self);
        [self.playerManager changeSubtitleWithIndex:selectedIndex];
    }];
    [self.subtitleView setShowSubtitleCallback:^(BOOL showSubtitle) {
        bjl_strongify(self);
        self.subtitleLabel.hidden = !showSubtitle;
    }];
    [self.videoContentContainer addSubview:self.subtitleView];
    [self.subtitleView bjl_makeConstraints:^(BJLConstraintMaker *_Nonnull make) {
        make.top.bottom.right.equalTo(self.videoContentContainer);
        make.width.equalTo(self.videoContentContainer).multipliedBy(0.27);
    }];
}

- (void)hideInterfaceViews {
    NSTimeInterval duration = 1.0;
    [self hideView:self.topBarView withDuration:duration];
    [self hideView:self.mediaControlView withDuration:duration];
    [self hideView:self.lockButton withDuration:duration];
    [self hideView:self.vSubtitleButton withDuration:duration];
}

- (void)updateConstriantsWithLayoutType:(BJVPlayerViewLayoutType)layoutType {
    [self _updateConstriantsWithLayoutType:layoutType];
}

- (void)updateWithPlayState:(BJVPlayerStatus)state {
    [self _updateWithPlayState:state];
}

- (void)updateVSubtitleButtonConstriants:(BOOL)subtitleExist {
    [self _updateVSubtitleButtonConstriants:subtitleExist];
}

- (void)audoAdjustPlayerViewContainerRatio {
    [self _audoAdjustPlayerViewContainerRatio];
}
- (void)setupCatalogueView:(NSArray<BJVVideoCatalogueItem *> *)list {
    [self _setupCatalogueView:list];
}

- (void)updateConstriantsWithTripartiteLayoutInfo:(BJVTripartiteLayoutInfo *)tripartiteLayoutInfo {
    [self _updateConstriantsWithTripartiteLayoutInfo:tripartiteLayoutInfo];
}

- (void)updateConstriantsForCommentsVC:(BOOL)enableVideoComment {
    [self _updateConstriantsForCommentsVC:enableVideoComment];
}

- (void)updatePlayerViewConstraintWithVideoRatio:(CGFloat)ratio {
    [self _updatePlayerViewConstraintWithVideoRatio:ratio];
}

@end
